@@ -21,11 +21,10 @@ import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import OrderedDict, hbytes
 from hypothesis.searchstrategy.strategies import SearchStrategy, \
-    MappedSearchStrategy, one_of_strategies
+    MappedSearchStrategy, combine_labels, one_of_strategies
 
 
 class TupleStrategy(SearchStrategy):
-
     """A strategy responsible for fixed length tuples based on heterogenous
     strategies for each of their elements."""
 
@@ -39,6 +38,10 @@ class TupleStrategy(SearchStrategy):
         for s in self.element_strategies:
             s.validate()
 
+    def calc_label(self):
+        return combine_labels(
+            self.class_label, *[s.label for s in self.element_strategies])
+
     def __repr__(self):
         if len(self.element_strategies) == 1:
             tuple_string = '%s,' % (repr(self.element_strategies[0]),)
@@ -68,7 +71,6 @@ TERMINATOR = hbytes(b'\0')
 
 
 class ListStrategy(SearchStrategy):
-
     """A strategy for lists which takes an intended average length and a
     strategy for each of its element types and generates lists containing any
     of those element types.
@@ -92,6 +94,9 @@ class ListStrategy(SearchStrategy):
         self.max_size = max_size or float('inf')
         self.element_strategy = one_of_strategies(strategies)
 
+    def calc_label(self):
+        return combine_labels(self.class_label, self.element_strategy.label)
+
     def do_validate(self):
         self.element_strategy.validate()
         if self.is_empty:
@@ -185,12 +190,11 @@ class UniqueListStrategy(SearchStrategy):
 
 
 class FixedKeysDictStrategy(MappedSearchStrategy):
-
     """A strategy which produces dicts with a fixed set of keys, given a
     strategy for each of their equivalent values.
 
-    e.g. {'foo' : some_int_strategy} would
-    generate dicts with the single key 'foo' mapping to some integer.
+    e.g. {'foo' : some_int_strategy} would generate dicts with the single
+    key 'foo' mapping to some integer.
 
     """
 
