@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -27,8 +27,8 @@ from hypothesis.searchstrategy.strategies import SearchStrategy, \
 
 
 class OneCharStringStrategy(SearchStrategy):
-
     """A strategy which generates single character strings of text type."""
+
     specifier = text_type
     zero_point = ord('0')
 
@@ -45,6 +45,7 @@ class OneCharStringStrategy(SearchStrategy):
             min_codepoint=min_codepoint,
             max_codepoint=max_codepoint,
             include_characters=whitelist_characters,
+            exclude_characters=blacklist_characters,
         )
         if not intervals:
             raise InvalidArgument(
@@ -55,32 +56,17 @@ class OneCharStringStrategy(SearchStrategy):
             self.whitelist_characters = set(whitelist_characters)
         else:
             self.whitelist_characters = set()
-        if blacklist_characters:
-            self.blacklist_characters = set(
-                b for b in blacklist_characters if ord(b) in self.intervals
-            )
-            if (len(self.whitelist_characters) == 0 and
-                    len(self.blacklist_characters) == len(self.intervals)):
-                raise InvalidArgument(
-                    'No valid characters in set'
-                )
-        else:
-            self.blacklist_characters = set()
         self.zero_point = self.intervals.index_above(ord('0'))
 
     def do_draw(self, data):
-        while True:
-            i = integer_range(
-                data, 0, len(self.intervals) - 1,
-                center=self.zero_point,
-            )
-            c = hunichr(self.intervals[i])
-            if c not in self.blacklist_characters:
-                return c
+        i = integer_range(
+            data, 0, len(self.intervals) - 1,
+            center=self.zero_point,
+        )
+        return hunichr(self.intervals[i])
 
 
 class StringStrategy(MappedSearchStrategy):
-
     """A strategy for text strings, defined in terms of a strategy for lists of
     single character text strings."""
 
@@ -97,7 +83,6 @@ class StringStrategy(MappedSearchStrategy):
 
 
 class BinaryStringStrategy(MappedSearchStrategy):
-
     """A strategy for strings of bytes, defined in terms of a strategy for
     lists of bytes."""
 
